@@ -1,4 +1,5 @@
 # Copyright 2011-2014 OpenStack Foundation,author: Min Wang,German Eichberger
+# Copyright 2015 Hewlett-Packard Development Company, L.P.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -22,7 +23,7 @@ class AmphoraLoadBalancerDriver(object):
 
     @abc.abstractmethod
     def update(self, listener, vip):
-        """Update the amphora with a new configuration
+        """Update the amphora with a new configuration.
 
         :param listener: listener object,
         need to use its protocol_port property
@@ -32,13 +33,13 @@ class AmphoraLoadBalancerDriver(object):
         :returns: return a value list (listener, vip, status flag--update)
 
         At this moment, we just build the basic structure for testing, will
-        add more function along with the development
+        add more function along with the development.
         """
         pass
 
     @abc.abstractmethod
     def stop(self, listener, vip):
-        """Stop the listener on the vip
+        """Stop the listener on the vip.
 
         :param listener: listener object,
         need to use its protocol_port property
@@ -48,13 +49,13 @@ class AmphoraLoadBalancerDriver(object):
         :returns: return a value list (listener, vip, status flag--suspend)
 
         At this moment, we just build the basic structure for testing, will
-        add more function along with the development
+        add more function along with the development.
         """
         pass
 
     @abc.abstractmethod
     def start(self, listener, vip):
-        """Start the listener on the vip
+        """Start the listener on the vip.
 
         :param listener: listener object,
         need to use its protocol_port property
@@ -64,13 +65,13 @@ class AmphoraLoadBalancerDriver(object):
         :returns: return a value list (listener, vip, status flag--enable)
 
         At this moment, we just build the basic structure for testing, will
-        add more function along with the development
+        add more function along with the development.
         """
         pass
 
     @abc.abstractmethod
     def delete(self, listener, vip):
-        """Delete the listener on the vip
+        """Delete the listener on the vip.
 
         :param listener: listener object,
         need to use its protocol_port property
@@ -80,13 +81,13 @@ class AmphoraLoadBalancerDriver(object):
         :returns: return a value list (listener, vip, status flag--delete)
 
         At this moment, we just build the basic structure for testing, will
-        add more function along with the development
+        add more function along with the development.
         """
         pass
 
     @abc.abstractmethod
     def get_info(self, amphora):
-        """Returns information about the amphora
+        """Returns information about the amphora.
 
         :param amphora: amphora object, need to use its id property
         :type amphora: object
@@ -103,7 +104,7 @@ class AmphoraLoadBalancerDriver(object):
 
     @abc.abstractmethod
     def get_diagnostics(self, amphora):
-        """Return ceilometer ready health
+        """Return ceilometer ready diagnostic data.
 
         :param amphora: amphora object, need to use its id property
         :type amphora: object
@@ -114,28 +115,27 @@ class AmphoraLoadBalancerDriver(object):
         add more function along with the development, eventually, we want it
         run some expensive self tests to determine if the amphora and the lbs
         are healthy the idea is that those tests are triggered more infrequent
-        than the health gathering
+        than the health gathering.
         """
         pass
 
     @abc.abstractmethod
     def finalize_amphora(self, amphora):
-        """It is called before listeners configured while amphora was built
+        """Finalize the amphora before any listeners are configured.
 
         :param amphora: amphora object, need to use its id property
         :type amphora: object
-        :returns: return a value list (amphora.id, status flag--'ge
-        t_diagnostics')
+        :returns: None
 
         At this moment, we just build the basic structure for testing, will
-        add more function along with the development, eventually, we want it
-        run some expensive self tests to determine if the amphora and the lbs
-        are healthy the idea is that those tests are triggered more infrequent
-        than the health gathering
+        add more function along with the development. This is a hook for
+        drivers who need to do additional work before an amphora becomes ready
+        to accept listeners. Please keep in mind that amphora might be kept in
+        an offline pool after this call.
         """
         pass
 
-    def post_vip_plug(self, load_balancer, amphorae_network_config):
+    def post_vip_plug(self, amphora, load_balancer, amphorae_network_config):
         """Called after network driver has allocated and plugged the VIP
 
         :param load_balancer: A load balancer that just had its vip allocated
@@ -146,6 +146,9 @@ class AmphoraLoadBalancerDriver(object):
                                         amphorae owns.
         :type vip_network: octavia.network.data_models.AmphoraNetworkConfig
         :returns: None
+
+        This is to do any additional work needed on the amphorae to plug
+        the vip, such as bring up interfaces.
         """
         pass
 
@@ -165,26 +168,36 @@ class AmphoraLoadBalancerDriver(object):
         pass
 
     def start_health_check(self, health_mixin):
-        """start check health
-
+        """Start health checks.
 
         :param health_mixin: health mixin object
         :type amphora: object
 
-        Start listener process and  calls HealthMixin to update
+        Starts listener process and calls HealthMixin to update
         databases information.
         """
         pass
 
     def stop_health_check(self):
-        """stop check health
+        """Stop health checks.
 
-
-        Stop listener process and  calls HealthMixin to update
+        Stops listener process and calls HealthMixin to update
         databases information.
         """
         pass
 
+    def upload_cert_amp(self, amphora, pem_file):
+        """Upload cert info to the amphora.
+
+        :param amphora: amphora object, needs id and network ip(s)
+        :type amphora: object
+        :param pem_file: a certificate file
+        :type pem_file: file object
+
+        Upload cert file to amphora for Controller Communication.
+        """
+        pass
+
 
 @six.add_metaclass(abc.ABCMeta)
 class HealthMixin(object):
@@ -229,3 +242,44 @@ class StatsMixin(object):
         awesome update code and code to send to ceilometer
         """
         pass
+
+
+@six.add_metaclass(abc.ABCMeta)
+class VRRPDriverMixin(object):
+    """Abstract mixin class for VRRP support in loadbalancer amphorae
+
+    Usage: To plug VRRP support in another service driver XYZ, use:
+    @plug_mixin(XYZ)
+    class XYZ: ...
+    """
+    @abc.abstractmethod
+    def update_vrrp_conf(self, loadbalancer):
+        """Update amphorae of the loadbalancer with a new VRRP configuration
+
+        :param loadbalancer: loadbalancer object
+        """
+        pass
+
+    @abc.abstractmethod
+    def stop_vrrp_service(self, loadbalancer):
+        """Stop the vrrp services running on the loadbalancer's amphorae
+
+        :param loadbalancer: loadbalancer object
+        """
+        pass
+
+    @abc.abstractmethod
+    def start_vrrp_service(self, loadbalancer):
+        """Start the VRRP services of all amphorae of the loadbalancer
+
+        :param loadbalancer: loadbalancer object
+        """
+        pass
+
+    @abc.abstractmethod
+    def reload_vrrp_service(self, loadbalancer):
+        """Reload the VRRP services of all amphorae of the loadbalancer
+
+        :param loadbalancer: loadbalancer object
+        """
+        pass
