@@ -17,11 +17,11 @@ import logging
 
 from oslo_config import cfg
 import six
-from stevedore import driver as stevedore_driver
 from taskflow import task
 from taskflow.types import failure
 
 from octavia.common import constants
+from octavia.common import utils
 from octavia.i18n import _LW, _LE
 from octavia.network import base
 from octavia.network import data_models as n_data_models
@@ -37,11 +37,7 @@ class BaseNetworkTask(task.Task):
     def __init__(self, **kwargs):
         super(BaseNetworkTask, self).__init__(**kwargs)
 
-        self.network_driver = stevedore_driver.DriverManager(
-            namespace='octavia.network.drivers',
-            name=CONF.controller_worker.network_driver,
-            invoke_on_load=True
-        ).driver
+        self.network_driver = utils.get_network_driver()
 
 
 class CalculateAmphoraDelta(BaseNetworkTask):
@@ -54,19 +50,16 @@ class CalculateAmphoraDelta(BaseNetworkTask):
         # Figure out what networks we want
         # seed with lb network(s)
         subnet = self.network_driver.get_subnet(loadbalancer.vip.subnet_id)
-        desired_network_ids = {CONF.controller_worker.amp_network,
-                               subnet.network_id}
+        # TODO(ptoohill): amp_network is deprecated, remove when ready...
+        desired_network_ids = {subnet.network_id}.union(
+            CONF.controller_worker.amp_boot_network_list)
+        if CONF.controller_worker.amp_network:
+            desired_network_ids.add(CONF.controller_worker.amp_network)
 
-        if not loadbalancer.listeners:
-            return None
-
-        for listener in loadbalancer.listeners:
-            if (not listener.default_pool) or (
-                    not listener.default_pool.members):
-                continue
+        for pool in loadbalancer.pools:
             member_networks = [
                 self.network_driver.get_subnet(member.subnet_id).network_id
-                for member in listener.default_pool.members
+                for member in pool.members
                 if member.subnet_id
             ]
             desired_network_ids.update(member_networks)
@@ -211,9 +204,13 @@ class GetMemberPorts(BaseNetworkTask):
                 continue
             port.network = self.network_driver.get_network(port.network_id)
             for fixed_ip in port.fixed_ips:
+                if amphora.lb_network_ip == fixed_ip.ip_address:
+                    break
                 fixed_ip.subnet = self.network_driver.get_subnet(
                     fixed_ip.subnet_id)
-            member_ports.append(port)
+            # Only add the port to the list if the IP wasn't the mgmt IP
+            else:
+                member_ports.append(port)
         return member_ports
 
 
@@ -288,6 +285,14 @@ class PlugVIP(BaseNetworkTask):
                     loadbalancer.id)
 
         try:
+            # Make sure we have the current port IDs for cleanup
+            for amp_data in result:
+                for amphora in six.moves.filter(
+                        lambda amp: amp.id == amp_data.id,
+                        loadbalancer.amphorae):
+                    amphora.vrrp_port_id = amp_data.vrrp_port_id
+                    amphora.ha_port_id = amp_data.ha_port_id
+
             self.network_driver.unplug_vip(loadbalancer, loadbalancer.vip)
         except Exception as e:
             LOG.error(_LE("Failed to unplug VIP.  Resources may still "
@@ -372,28 +377,7 @@ class GetAmphoraeNetworkConfigs(BaseNetworkTask):
 
     def execute(self, loadbalancer):
         LOG.debug("Retrieving vip network details.")
-        vip_subnet = self.network_driver.get_subnet(loadbalancer.vip.subnet_id)
-        vip_port = self.network_driver.get_port(loadbalancer.vip.port_id)
-        amp_net_configs = {}
-        for amp in loadbalancer.amphorae:
-            if amp.status != constants.DELETED:
-                LOG.debug("Retrieving network details for amphora %s", amp.id)
-                vrrp_port = self.network_driver.get_port(amp.vrrp_port_id)
-                vrrp_subnet = self.network_driver.get_subnet(
-                    vrrp_port.get_subnet_id(amp.vrrp_ip))
-                ha_port = self.network_driver.get_port(amp.ha_port_id)
-                ha_subnet = self.network_driver.get_subnet(
-                    ha_port.get_subnet_id(amp.ha_ip))
-                amp_net_configs[amp.id] = n_data_models.AmphoraNetworkConfig(
-                    amphora=amp,
-                    vip_subnet=vip_subnet,
-                    vip_port=vip_port,
-                    vrrp_subnet=vrrp_subnet,
-                    vrrp_port=vrrp_port,
-                    ha_subnet=ha_subnet,
-                    ha_port=ha_port
-                )
-        return amp_net_configs
+        return self.network_driver.get_network_configs(loadbalancer)
 
 
 class FailoverPreparationForAmphora(BaseNetworkTask):
@@ -439,3 +423,34 @@ class PlugPorts(BaseNetworkTask):
                       '{compute_id}.'.format(port_id=port.id,
                                              compute_id=amphora.compute_id))
             self.network_driver.plug_port(amphora, port)
+
+
+class PlugVIPPort(BaseNetworkTask):
+    """Task to plug a VIP into a compute instance."""
+
+    def execute(self, amphora, amphorae_network_config):
+        vrrp_port = amphorae_network_config.get(amphora.id).vrrp_port
+        LOG.debug('Plugging VIP VRRP port ID: {port_id} into compute '
+                  'instance: {compute_id}.'.format(
+                      port_id=vrrp_port.id, compute_id=amphora.compute_id))
+        self.network_driver.plug_port(amphora, vrrp_port)
+
+    def revert(self, result, amphora, amphorae_network_config,
+               *args, **kwargs):
+        vrrp_port = None
+        try:
+            vrrp_port = amphorae_network_config.get(amphora.id).vrrp_port
+            self.network_driver.unplug_port(amphora, vrrp_port)
+        except Exception:
+            LOG.warning(_LW('Failed to unplug vrrp port: {port} '
+                            'from amphora: {amp}').format(port=vrrp_port.id,
+                                                          amp=amphora.id))
+
+
+class WaitForPortDetach(BaseNetworkTask):
+    """Task to wait for the neutron ports to detach from an amphora."""
+
+    def execute(self, amphora):
+        LOG.debug('Waiting for ports to detach from amphora: '
+                  '{amp_id}.'.format(amp_id=amphora.id))
+        self.network_driver.wait_for_port_detach(amphora)
