@@ -35,7 +35,7 @@ class OctaviaBase(models.ModelBase):
         elif obj.__class__.__name__ in ['SessionPersistence', 'HealthMonitor']:
             return obj.__class__.__name__ + obj.pool_id
         elif obj.__class__.__name__ in ['ListenerStatistics']:
-            return obj.__class__.__name__ + obj.listener_id
+            return obj.__class__.__name__ + obj.listener_id + obj.amphora_id
         elif obj.__class__.__name__ in ['VRRPGroup', 'Vip']:
             return obj.__class__.__name__ + obj.load_balancer_id
         elif obj.__class__.__name__ in ['AmphoraHealth']:
@@ -64,6 +64,7 @@ class OctaviaBase(models.ModelBase):
         dm_kwargs = {}
         for column in self.__table__.columns:
             dm_kwargs[column.name] = getattr(self, column.name)
+
         attr_names = [attr_name for attr_name in dir(self)
                       if not attr_name.startswith('_')]
         # Appending early, as any unique ID should be defined already and
