@@ -25,9 +25,12 @@ from octavia.common import constants
 from octavia.common import data_models
 from octavia.i18n import _LE, _LI, _LW
 from octavia.network import base
+from octavia.network import data_models as n_data_models
 from octavia.network.drivers.neutron import base as neutron_base
 from octavia.network.drivers.neutron import utils
 
+import ipaddress
+
 
 LOG = logging.getLogger(__name__)
 AAP_EXT_ALIAS = 'allowed-address-pairs'
@@ -55,8 +58,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         )
 
     def _check_aap_loaded(self):
-        aliases = [ext.get('alias') for ext in self._extensions]
-        if AAP_EXT_ALIAS not in aliases:
+        if not self._check_extension_enabled(AAP_EXT_ALIAS):
             raise base.NetworkException(
                 'The {alias} extension is not enabled in neutron.  This '
                 'driver cannot be used with the {alias} extension '
@@ -81,13 +83,24 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             if interface.network_id == network_id:
                 return interface
 
-    def _plug_amphora_vip(self, compute_id, network_id):
+    def _plug_amphora_vip(self, amphora, network_id):
+        # We need a vip port owned by Octavia for Act/Stby and failover
         try:
-            interface = self.plug_network(compute_id, network_id)
+            port = {'port': {'name': 'octavia-lb-vrrp-' + amphora.id,
+                             'network_id': network_id,
+                             'admin_state_up': True,
+                             'device_owner': OCTAVIA_OWNER}}
+            new_port = self.neutron_client.create_port(port)
+            new_port = utils.convert_port_dict_to_model(new_port)
+
+            LOG.debug('Created vip port: {port_id} for amphora: {amp}'.format(
+                port_id=new_port.id, amp=amphora.id))
+
+            interface = self.plug_port(amphora, new_port)
         except Exception:
             message = _LE('Error plugging amphora (compute_id: {compute_id}) '
                           'into vip network {network_id}.').format(
-                              compute_id=compute_id,
+                              compute_id=amphora.compute_id,
                               network_id=network_id)
             LOG.exception(message)
             raise base.PlugVIPException(message)
@@ -111,6 +124,11 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         if sec_grps and sec_grps.get('security_groups'):
             return sec_grps.get('security_groups')[0]
 
+    def _get_ethertype_for_ip(self, ip):
+        address = ipaddress.ip_address(
+            ip if six.text_type == type(ip) else six.u(ip))
+        return 'IPv6' if address.version is 6 else 'IPv4'
+
     def _update_security_group_rules(self, load_balancer, sec_grp_id):
         rules = self.neutron_client.list_security_group_rules(
             security_group_id=sec_grp_id)
@@ -140,9 +158,11 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             if rule.get('port_range_max') in del_ports:
                 self.neutron_client.delete_security_group_rule(rule.get('id'))
 
+        ethertype = self._get_ethertype_for_ip(load_balancer.vip.ip_address)
         for port in add_ports:
             self._create_security_group_rule(sec_grp_id, 'TCP', port_min=port,
-                                             port_max=port)
+                                             port_max=port,
+                                             ethertype=ethertype)
 
         # Currently we are using the VIP network for VRRP
         # so we need to open up the protocols for it
@@ -152,7 +172,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 self._create_security_group_rule(
                     sec_grp_id,
                     constants.VRRP_PROTOCOL_NUM,
-                    direction='ingress')
+                    direction='ingress',
+                    ethertype=ethertype)
             except neutron_client_exceptions.Conflict:
                 # It's ok if this rule already exists
                 pass
@@ -162,7 +183,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             try:
                 self._create_security_group_rule(
                     sec_grp_id, constants.AUTH_HEADER_PROTOCOL_NUMBER,
-                    direction='ingress')
+                    direction='ingress', ethertype=ethertype)
             except neutron_client_exceptions.Conflict:
                 # It's ok if this rule already exists
                 pass
@@ -175,14 +196,17 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             sec_grp_name = VIP_SECURITY_GRP_PREFIX + load_balancer.id
             sec_grp = self._create_security_group(sec_grp_name)
         self._update_security_group_rules(load_balancer, sec_grp.get('id'))
-        self._add_vip_security_group_to_port(load_balancer.id, vip.port_id)
+        self._add_vip_security_group_to_port(load_balancer.id, vip.port_id,
+                                             sec_grp.get('id'))
 
-    def _add_vip_security_group_to_port(self, load_balancer_id, port_id):
-        sec_grp = self._get_lb_security_group(load_balancer_id)
+    def _add_vip_security_group_to_port(self, load_balancer_id, port_id,
+                                        sec_grp_id=None):
+        sec_grp_id = (sec_grp_id or
+                      self._get_lb_security_group(load_balancer_id).get('id'))
         try:
-            self._add_security_group_to_port(sec_grp.get('id'), port_id)
-        except base.PortNotFound as e:
-            raise e
+            self._add_security_group_to_port(sec_grp_id, port_id)
+        except base.PortNotFound:
+            raise
         except base.NetworkException as e:
             raise base.PlugVIPException(str(e))
 
@@ -213,14 +237,34 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         LOG.exception(message)
         raise base.DeallocateVIPException(message)
 
-    def deallocate_vip(self, vip):
-        # Delete the vrrp_port (instance port) in case nova didn't
-        # This can happen if a failover has occurred.
-        try:
-            for amphora in six.moves.filter(
-                lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
-                    vip.load_balancer.amphorae):
+    @staticmethod
+    def _filter_amphora(amp):
+        return amp.status == constants.AMPHORA_ALLOCATED
 
+    def _delete_security_group(self, vip, port):
+        if self.sec_grp_enabled:
+            sec_grp = self._get_lb_security_group(vip.load_balancer.id)
+            if sec_grp:
+                sec_grp = sec_grp.get('id')
+                LOG.info(
+                    _LI("Removing security group %(sg)s from port %(port)s"),
+                    {'sg': sec_grp, 'port': vip.port_id})
+                raw_port = self.neutron_client.show_port(port.id)
+                sec_grps = raw_port.get('port', {}).get('security_groups', [])
+                if sec_grp in sec_grps:
+                    sec_grps.remove(sec_grp)
+                port_update = {'port': {'security_groups': sec_grps}}
+                self.neutron_client.update_port(port.id, port_update)
+                self._delete_vip_security_group(sec_grp)
+
+    def deallocate_vip(self, vip):
+        """Delete the vrrp_port (instance port) in case nova didn't
+
+        This can happen if a failover has occurred.
+        """
+        try:
+            for amphora in six.moves.filter(self._filter_amphora,
+                                            vip.load_balancer.amphorae):
                 self.neutron_client.delete_port(amphora.vrrp_port_id)
         except (neutron_client_exceptions.NotFound,
                 neutron_client_exceptions.PortNotFoundClient):
@@ -233,34 +277,20 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             msg = ("Can't deallocate VIP because the vip port {0} cannot be "
                    "found in neutron".format(vip.port_id))
             raise base.VIPConfigurationNotFound(msg)
-        if port.device_owner != OCTAVIA_OWNER:
+
+        self._delete_security_group(vip, port)
+
+        if port.device_owner == OCTAVIA_OWNER:
+            try:
+                self.neutron_client.delete_port(vip.port_id)
+            except Exception:
+                message = _LE('Error deleting VIP port_id {port_id} from '
+                              'neutron').format(port_id=vip.port_id)
+                LOG.exception(message)
+                raise base.DeallocateVIPException(message)
+        else:
             LOG.info(_LI("Port %s will not be deleted by Octavia as it was "
                          "not created by Octavia."), vip.port_id)
-            if self.sec_grp_enabled:
-                sec_grp = self._get_lb_security_group(vip.load_balancer.id)
-                sec_grp = sec_grp.get('id')
-                LOG.info(
-                    _LI("Removing security group %(sg)s from port %(port)s"),
-                    {'sg': sec_grp, 'port': vip.port_id})
-                raw_port = self.neutron_client.show_port(port.id)
-                sec_grps = raw_port.get('port', {}).get('security_groups', [])
-                if sec_grp in sec_grps:
-                    sec_grps.remove(sec_grp)
-                port_update = {'port': {'security_groups': sec_grps}}
-                self.neutron_client.update_port(port.id, port_update)
-                self._delete_vip_security_group(sec_grp)
-            return
-        try:
-            self.neutron_client.delete_port(vip.port_id)
-        except Exception:
-            message = _LE('Error deleting VIP port_id {port_id} from '
-                          'neutron').format(port_id=vip.port_id)
-            LOG.exception(message)
-            raise base.DeallocateVIPException(message)
-        if self.sec_grp_enabled:
-            sec_grp = self._get_lb_security_group(vip.load_balancer.id)
-            sec_grp = sec_grp.get('id')
-            self._delete_vip_security_group(sec_grp)
 
     def plug_vip(self, load_balancer, vip):
         if self.sec_grp_enabled:
@@ -274,8 +304,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             interface = self._get_plugged_interface(amphora.compute_id,
                                                     subnet.network_id)
             if not interface:
-                interface = self._plug_amphora_vip(amphora.compute_id,
-                                                   subnet.network_id)
+                interface = self._plug_amphora_vip(amphora, subnet.network_id)
+
             self._add_vip_address_pair(interface.port_id, vip.ip_address)
             if self.sec_grp_enabled:
                 self._add_vip_security_group_to_port(load_balancer.id,
@@ -329,8 +359,9 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         try:
             subnet = self.get_subnet(vip.subnet_id)
         except base.SubnetNotFound:
-            msg = ("Can't unplug vip because vip subnet {0} was not "
-                   "found").format(vip.subnet_id)
+            msg = _LE("Can't unplug vip because vip subnet {0} was not "
+                      "found").format(vip.subnet_id)
+            LOG.exception(msg)
             raise base.PluggedVIPNotFound(msg)
         for amphora in six.moves.filter(
             lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
@@ -341,6 +372,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             if not interface:
                 # Thought about raising PluggedVIPNotFound exception but
                 # then that wouldn't evaluate all amphorae, so just continue
+                LOG.debug(_LI('Cannot get amphora %s interface, skipped'),
+                          amphora.compute_id)
                 continue
             try:
                 self.unplug_network(amphora.compute_id, subnet.network_id)
@@ -359,6 +392,19 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 LOG.exception(message)
                 raise base.UnplugVIPException(message)
 
+            # Delete the VRRP port if we created it
+            try:
+                port = self.get_port(amphora.vrrp_port_id)
+                if port.name.startswith('octavia-lb-vrrp-'):
+                    self.neutron_client.delete_port(amphora.vrrp_port_id)
+            except base.PortNotFound:
+                pass
+            except Exception as e:
+                LOG.error(_LE('Failed to delete port.  Resources may still '
+                              'be in use for port: %(port)s due to '
+                              'error: %s(except)s'),
+                          {'port': amphora.vrrp_port_id, 'except': e})
+
     def plug_network(self, compute_id, network_id, ip_address=None):
         try:
             interface = self.nova_client.servers.interface_attach(
@@ -432,27 +478,23 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
         for port in ports:
             try:
-                self.nova_client.servers.interface_detach(
-                    server=amphora.compute_id, port=port.id)
-            except Exception as e:
-                LOG.warning(_LW('Failed to detach port %(portid)s from '
-                                'instance %(compid)s.  Error: '
-                                '%(error)s') % {'portid': port.id,
-                                                'compid': amphora.compute_id,
-                                                'error': str(e)})
-            try:
-                self.neutron_client.update_port(port.id,
-                                                {'port': {'device_id': '',
-                                                          'dns_name': ''}})
+
+                if self.dns_integration_enabled:
+                    self.neutron_client.update_port(port.id,
+                                                    {'port': {'dns_name': ''}})
+
             except (neutron_client_exceptions.NotFound,
                     neutron_client_exceptions.PortNotFoundClient):
                 raise base.PortNotFound()
 
     def plug_port(self, amphora, port):
+        plugged_interface = None
         try:
-            self.nova_client.servers.interface_attach(
+            interface = self.nova_client.servers.interface_attach(
                 server=amphora.compute_id, net_id=None,
                 fixed_ip=None, port_id=port.id)
+            plugged_interface = self._nova_interface_to_octavia_interface(
+                amphora.compute_id, interface)
         except nova_client_exceptions.NotFound as e:
             if 'Instance' in e.message:
                 raise base.AmphoraNotFound(e.message)
@@ -463,6 +505,11 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         except nova_client_exceptions.Conflict:
             LOG.info(_LI('Port %(portid)s is already plugged, '
                      'skipping') % {'portid': port.id})
+            plugged_interface = n_data_models.Interface(
+                compute_id=amphora.compute_id,
+                network_id=port.network_id,
+                port_id=port.id,
+                fixed_ips=port.fixed_ips)
         except Exception:
             message = _LE('Error plugging amphora (compute_id: '
                           '{compute_id}) into port '
@@ -471,3 +518,81 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                               port_id=port.id)
             LOG.exception(message)
             raise base.PlugNetworkException(message)
+
+        return plugged_interface
+
+    def get_network_configs(self, loadbalancer):
+        vip_subnet = self.get_subnet(loadbalancer.vip.subnet_id)
+        vip_port = self.get_port(loadbalancer.vip.port_id)
+        amp_configs = {}
+        for amp in loadbalancer.amphorae:
+            if amp.status != constants.DELETED:
+                LOG.debug("Retrieving network details for amphora %s", amp.id)
+                vrrp_port = self.get_port(amp.vrrp_port_id)
+                vrrp_subnet = self.get_subnet(
+                    vrrp_port.get_subnet_id(amp.vrrp_ip))
+                ha_port = self.get_port(amp.ha_port_id)
+                ha_subnet = self.get_subnet(
+                    ha_port.get_subnet_id(amp.ha_ip))
+
+                amp_configs[amp.id] = n_data_models.AmphoraNetworkConfig(
+                    amphora=amp,
+                    vip_subnet=vip_subnet,
+                    vip_port=vip_port,
+                    vrrp_subnet=vrrp_subnet,
+                    vrrp_port=vrrp_port,
+                    ha_subnet=ha_subnet,
+                    ha_port=ha_port
+                )
+        return amp_configs
+
+    def wait_for_port_detach(self, amphora):
+        """Waits for the amphora ports device_id to be unset.
+
+        This method waits for the ports on an amphora device_id
+        parameter to be '' or None which signifies that nova has
+        finished detaching the port from the instance.
+
+        :param amphora: Amphora to wait for ports to detach.
+        :returns: None
+        :raises TimeoutException: Port did not detach in interval.
+        :raises PortNotFound: Port was not found by neutron.
+        """
+        interfaces = self.get_plugged_networks(compute_id=amphora.compute_id)
+
+        ports = []
+        port_detach_timeout = CONF.networking.port_detach_timeout
+        for interface_ in interfaces:
+            port = self.get_port(port_id=interface_.port_id)
+            ips = port.fixed_ips
+            lb_network = False
+            for ip in ips:
+                if ip.ip_address == amphora.lb_network_ip:
+                    lb_network = True
+            if not lb_network:
+                ports.append(port)
+
+        for port in ports:
+            try:
+                neutron_port = self.neutron_client.show_port(
+                    port.id).get('port')
+                device_id = neutron_port['device_id']
+                start = int(time.time())
+
+                while device_id:
+                    time.sleep(CONF.networking.retry_interval)
+                    neutron_port = self.neutron_client.show_port(
+                        port.id).get('port')
+                    device_id = neutron_port['device_id']
+
+                    timed_out = int(time.time()) - start >= port_detach_timeout
+
+                    if device_id and timed_out:
+                        message = ('Port %s failed to detach (device_id %s) '
+                                   'within the required time (%s s).' %
+                                   (port.id, device_id, port_detach_timeout))
+                        raise base.TimeoutException(message)
+
+            except (neutron_client_exceptions.NotFound,
+                    neutron_client_exceptions.PortNotFoundClient):
+                    pass
