@@ -13,35 +13,58 @@
 #    under the License.
 
 from neutronclient.common import exceptions as neutron_client_exceptions
+from oslo_config import cfg
 from oslo_log import log as logging
 
 from octavia.common import clients
 from octavia.common import data_models
-from octavia.i18n import _LE, _LI
+from octavia.i18n import _LE
 from octavia.network import base
 from octavia.network import data_models as network_models
 from octavia.network.drivers.neutron import utils
 
 
 LOG = logging.getLogger(__name__)
+DNS_INT_EXT_ALIAS = 'dns-integration'
 SEC_GRP_EXT_ALIAS = 'security-group'
 
+CONF = cfg.CONF
+CONF.import_group('neutron', 'octavia.common.config')
+
 
 class BaseNeutronDriver(base.AbstractNetworkDriver):
 
-    def __init__(self, region=None):
-        self.sec_grp_enabled = True
-        self.neutron_client = clients.NeutronAuth.get_neutron_client(region)
-        extensions = self.neutron_client.list_extensions()
-        self._extensions = extensions.get('extensions')
-        self._check_sec_grps()
+    def __init__(self):
+        self.neutron_client = clients.NeutronAuth.get_neutron_client(
+            endpoint=CONF.neutron.endpoint,
+            region=CONF.neutron.region_name,
+            endpoint_type=CONF.neutron.endpoint_type,
+            service_name=CONF.neutron.service_name,
+            insecure=CONF.neutron.insecure,
+            ca_cert=CONF.neutron.ca_certificates_file
+        )
+        self._check_extension_cache = {}
+        self.sec_grp_enabled = self._check_extension_enabled(SEC_GRP_EXT_ALIAS)
+        self.dns_integration_enabled = self._check_extension_enabled(
+            DNS_INT_EXT_ALIAS)
 
-    def _check_sec_grps(self):
-        aliases = [ext.get('alias') for ext in self._extensions]
-        if SEC_GRP_EXT_ALIAS not in aliases:
-            LOG.info(_LI('Neutron security groups are disabled.  This driver'
-                         'will not manage any security groups.'))
-            self.sec_grp_enabled = False
+    def _check_extension_enabled(self, extension_alias):
+        if extension_alias in self._check_extension_cache:
+            status = self._check_extension_cache[extension_alias]
+            LOG.debug('Neutron extension {ext} cached as {status}'.format(
+                ext=extension_alias,
+                status='enabled' if status else 'disabled'))
+        else:
+            try:
+                self.neutron_client.show_extension(extension_alias)
+                LOG.debug('Neutron extension {ext} found enabled'.format(
+                    ext=extension_alias))
+                self._check_extension_cache[extension_alias] = True
+            except neutron_client_exceptions.NotFound:
+                LOG.debug('Neutron extension {ext} is not enabled'.format(
+                    ext=extension_alias))
+                self._check_extension_cache[extension_alias] = False
+        return self._check_extension_cache[extension_alias]
 
     def _port_to_vip(self, port, load_balancer):
         fixed_ip = None
@@ -52,6 +75,7 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
         return data_models.Vip(ip_address=fixed_ip.ip_address,
                                subnet_id=fixed_ip.subnet_id,
                                port_id=port.id,
+                               load_balancer=load_balancer,
                                load_balancer_id=load_balancer.id)
 
     def _nova_interface_to_octavia_interface(self, compute_id, nova_interface):
@@ -96,14 +120,15 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
 
     def _create_security_group_rule(self, sec_grp_id, protocol,
                                     direction='ingress', port_min=None,
-                                    port_max=None):
+                                    port_max=None, ethertype='IPv6'):
         rule = {
             'security_group_rule': {
                 'security_group_id': sec_grp_id,
                 'direction': direction,
                 'protocol': protocol,
                 'port_range_min': port_min,
-                'port_range_max': port_max
+                'port_range_max': port_max,
+                'ethertype': ethertype,
             }
         }
         self.neutron_client.create_security_group_rule(rule)
@@ -113,60 +138,72 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
         try:
             ports = self.neutron_client.list_ports(device_id=compute_id)
         except Exception:
-            message = ('Error retrieving plugged networks for compute '
-                       'device {compute_id}.').format(compute_id=compute_id)
-            LOG.debug(message)
+            LOG.debug('Error retrieving plugged networks for compute '
+                      'device %s.', compute_id)
             ports = {'ports': []}
         return [self._port_to_octavia_interface(
                 compute_id, port) for port in ports['ports']]
 
-    def get_network(self, network_id):
+    def _get_resource(self, resource_type, resource_id):
         try:
-            network = self.neutron_client.show_network(network_id)
-            return utils.convert_network_dict_to_model(network)
+            resource = getattr(self.neutron_client, 'show_%s' %
+                               resource_type)(resource_id)
+            return getattr(utils, 'convert_%s_dict_to_model' %
+                           resource_type)(resource)
         except neutron_client_exceptions.NotFound:
-            message = _LE('Network not found '
-                          '(network id: {network_id}.').format(
-                network_id=network_id)
-            LOG.exception(message)
-            raise base.NetworkNotFound(message)
+            message = _LE('{resource_type} not found '
+                          '({resource_type} id: {resource_id}.').format(
+                resource_type=resource_type, resource_id=resource_id)
+            raise getattr(base, '%sNotFound' %
+                          resource_type.capitalize())(message)
         except Exception:
-            message = _LE('Error retrieving network '
-                          '(network id: {network_id}.').format(
-                network_id=network_id)
+            message = _LE('Error retrieving {resource_type} '
+                          '({resource_type} id: {resource_id}.').format(
+                resource_type=resource_type, resource_id=resource_id)
             LOG.exception(message)
             raise base.NetworkException(message)
 
+    def _get_resource_by_filters(self, resource_type, **filters):
+        # Filter items are unique, return the filtered item from the list.
+        try:
+            resource = getattr(self.neutron_client, 'list_%ss' %
+                               resource_type)(**filters)
+            resource = resource['%ss' % resource_type][0]
+            return getattr(utils, 'convert_%s_dict_to_model' %
+                           resource_type)(resource)
+        except neutron_client_exceptions.NotFound:
+            message = _LE('{resource_type} not found '
+                          '({resource_type} Filters: {filters}.').format(
+                resource_type=resource_type, filters=filters)
+            LOG.exception(message)
+            raise getattr(base, '%sNotFound' %
+                          resource_type.capitalize())(message)
+        except Exception:
+            message = _LE('Error retrieving {resource_type} '
+                          '({resource_type} Filters: {filters}.').format(
+                resource_type=resource_type, filters=filters)
+            LOG.exception(message)
+            raise base.NetworkException(message)
+
+    def get_network(self, network_id):
+        return self._get_resource('network', network_id)
+
     def get_subnet(self, subnet_id):
-        try:
-            subnet = self.neutron_client.show_subnet(subnet_id)
-            return utils.convert_subnet_dict_to_model(subnet)
-        except neutron_client_exceptions.NotFound:
-            message = _LE('Subnet not found '
-                          '(subnet id: {subnet_id}.').format(
-                subnet_id=subnet_id)
-            LOG.exception(message)
-            raise base.SubnetNotFound(message)
-        except Exception:
-            message = _LE('Error retrieving subnet '
-                          '(subnet id: {subnet_id}.').format(
-                subnet_id=subnet_id)
-            LOG.exception(message)
-            raise base.NetworkException(message)
+        return self._get_resource('subnet', subnet_id)
 
     def get_port(self, port_id):
-        try:
-            port = self.neutron_client.show_port(port_id)
-            return utils.convert_port_dict_to_model(port)
-        except neutron_client_exceptions.NotFound:
-            message = _LE('Port not found '
-                          '(port id: {port_id}.').format(
-                port_id=port_id)
-            LOG.exception(message)
-            raise base.PortNotFound(message)
-        except Exception:
-            message = _LE('Error retrieving port '
-                          '(port id: {port_id}.').format(
-                port_id=port_id)
-            LOG.exception(message)
-            raise base.NetworkException(message)
+        return self._get_resource('port', port_id)
+
+    def get_network_by_name(self, network_name):
+        return self._get_resource_by_filters('network', name=network_name)
+
+    def get_subnet_by_name(self, subnet_name):
+        return self._get_resource_by_filters('subnet', name=subnet_name)
+
+    def get_port_by_name(self, port_name):
+        return self._get_resource_by_filters('port', name=port_name)
+
+    def get_port_by_net_id_device_id(self, network_id, device_id):
+        return self._get_resource_by_filters('port',
+                                             network_id=network_id,
+                                             device_id=device_id)
