@@ -14,6 +14,8 @@
 # under the License.
 
 import mock
+from oslo_config import cfg
+from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
 import requests_mock
 import six
@@ -26,21 +28,24 @@ from octavia.network import data_models as network_models
 from octavia.tests.unit import base as base
 from octavia.tests.unit.common.sample_configs import sample_configs
 
-FAKE_CIDR = '10.0.0.0/24'
-FAKE_GATEWAY = '10.0.0.1'
+FAKE_CIDR = '198.51.100.0/24'
+FAKE_GATEWAY = '192.51.100.1'
 FAKE_IP = 'fake'
 FAKE_PEM_FILENAME = "file_name"
-FAKE_SUBNET_INFO = {'subnet_cidr': FAKE_CIDR,
-                    'gateway': FAKE_GATEWAY,
-                    'mac_address': '123'}
 FAKE_UUID_1 = uuidutils.generate_uuid()
 FAKE_VRRP_IP = '10.1.0.1'
+FAKE_MAC_ADDRESS = '123'
 
 
 class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
 
     def setUp(self):
         super(TestHaproxyAmphoraLoadBalancerDriverTest, self).setUp()
+
+        DEST1 = '198.51.100.0/24'
+        DEST2 = '203.0.113.0/24'
+        NEXTHOP = '192.0.2.1'
+
         self.driver = driver.HaproxyAmphoraLoadBalancerDriver()
 
         self.driver.cert_manager = mock.MagicMock()
@@ -53,7 +58,23 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.amp = self.sl.load_balancer.amphorae[0]
         self.sv = sample_configs.sample_vip_tuple()
         self.lb = self.sl.load_balancer
-        self.port = network_models.Port(mac_address='123')
+        self.fixed_ip = mock.MagicMock()
+        self.fixed_ip.ip_address = '198.51.100.5'
+        self.fixed_ip.subnet.cidr = '198.51.100.0/24'
+        self.port = network_models.Port(mac_address=FAKE_MAC_ADDRESS,
+                                        fixed_ips=[self.fixed_ip])
+
+        self.host_routes = [network_models.HostRoute(destination=DEST1,
+                                                     nexthop=NEXTHOP),
+                            network_models.HostRoute(destination=DEST2,
+                                                     nexthop=NEXTHOP)]
+        host_routes_data = [{'destination': DEST1, 'nexthop': NEXTHOP},
+                            {'destination': DEST2, 'nexthop': NEXTHOP}]
+        self.subnet_info = {'subnet_cidr': FAKE_CIDR,
+                            'gateway': FAKE_GATEWAY,
+                            'mac_address': FAKE_MAC_ADDRESS,
+                            'vrrp_ip': self.amp.vrrp_ip,
+                            'host_routes': host_routes_data}
 
     @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
     @mock.patch('octavia.common.tls_utils.cert_parser.get_host_names')
@@ -80,12 +101,12 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.driver.client.get_cert_md5sum.assert_called_with(
             self.amp, self.sl.id, 'aFakeCN.pem')
         # this is called three times (last MD5 matches)
-        fp1 = ('--imainter1--\n\n--imainter1too--\n'
-               '\n--imapem1--\n\n--imakey1--\n')
-        fp2 = ('--imainter2--\n\n--imainter2too--\n'
-               '\n--imapem2--\n\n--imakey2--\n')
-        fp3 = ('--imainter3--\n\n--imainter3too--\n'
-               '\n--imapem3--\n\n--imakey3--\n')
+        fp1 = ('--imapem1--\n\n--imakey1--\n'
+               '\n--imainter1--\n\n--imainter1too--\n')
+        fp2 = ('--imapem2--\n\n--imakey2--\n'
+               '\n--imainter2--\n\n--imainter2too--\n')
+        fp3 = ('--imapem3--\n\n--imakey3--\n'
+               '\n--imainter3--\n\n--imainter3too--\n')
         ucp_calls = [
             mock.call(self.amp, self.sl.id, 'aFakeCN.pem', fp1),
             mock.call(self.amp, self.sl.id, 'aFakeCN.pem', fp2),
@@ -101,18 +122,6 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.driver.client.reload_listener.assert_called_once_with(
             self.amp, self.sl.id)
 
-        # listener down
-        self.driver.client.get_cert_md5sum.side_effect = [
-            'd41d8cd98f00b204e9800998ecf8427e'] * 3
-        self.driver.jinja.build_config.side_effect = ['fake_config']
-        self.driver.client.get_listener_status.side_effect = [
-            dict(status='BLAH')]
-
-        self.driver.update(self.sl, self.sv)
-
-        self.driver.client.start_listener.assert_called_once_with(
-            self.amp, self.sl.id)
-
     def test_upload_cert_amp(self):
         self.driver.upload_cert_amp(self.amp, six.b('test'))
         self.driver.client.update_cert_for_rotation.assert_called_once_with(
@@ -149,15 +158,63 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         amphorae_network_config = mock.MagicMock()
         amphorae_network_config.get().vip_subnet.cidr = FAKE_CIDR
         amphorae_network_config.get().vip_subnet.gateway_ip = FAKE_GATEWAY
+        amphorae_network_config.get().vip_subnet.host_routes = self.host_routes
         amphorae_network_config.get().vrrp_port = self.port
-        self.driver.post_vip_plug(self.lb, amphorae_network_config)
+        self.driver.post_vip_plug(self.amp, self.lb, amphorae_network_config)
         self.driver.client.plug_vip.assert_called_once_with(
-            self.amp, self.lb.vip.ip_address, FAKE_SUBNET_INFO)
+            self.amp, self.lb.vip.ip_address, self.subnet_info)
 
     def test_post_network_plug(self):
+        # Test dhcp path
+        port = network_models.Port(mac_address=FAKE_MAC_ADDRESS, fixed_ips=[])
+        self.driver.post_network_plug(self.amp, port)
+        self.driver.client.plug_network.assert_called_once_with(
+            self.amp, dict(mac_address=FAKE_MAC_ADDRESS, fixed_ips=[]))
+
+        self.driver.client.plug_network.reset_mock()
+
+        # Test fixed IP path
         self.driver.post_network_plug(self.amp, self.port)
         self.driver.client.plug_network.assert_called_once_with(
-            self.amp, dict(mac_address='123'))
+            self.amp, dict(mac_address=FAKE_MAC_ADDRESS,
+                           fixed_ips=[dict(ip_address='198.51.100.5',
+                                           subnet_cidr='198.51.100.0/24',
+                                           host_routes=[])]))
+
+    def test_post_network_plug_with_host_routes(self):
+        SUBNET_ID = 'SUBNET_ID'
+        FIXED_IP1 = '192.0.2.2'
+        FIXED_IP2 = '192.0.2.3'
+        SUBNET_CIDR = '192.0.2.0/24'
+        DEST1 = '198.51.100.0/24'
+        DEST2 = '203.0.113.0/24'
+        NEXTHOP = '192.0.2.1'
+        host_routes = [network_models.HostRoute(destination=DEST1,
+                                                nexthop=NEXTHOP),
+                       network_models.HostRoute(destination=DEST2,
+                                                nexthop=NEXTHOP)]
+        subnet = network_models.Subnet(id=SUBNET_ID, cidr=SUBNET_CIDR,
+                                       ip_version=4, host_routes=host_routes)
+        fixed_ips = [
+            network_models.FixedIP(subnet_id=subnet.id, ip_address=FIXED_IP1,
+                                   subnet=subnet),
+            network_models.FixedIP(subnet_id=subnet.id, ip_address=FIXED_IP2,
+                                   subnet=subnet)
+        ]
+        port = network_models.Port(mac_address=FAKE_MAC_ADDRESS,
+                                   fixed_ips=fixed_ips)
+        self.driver.post_network_plug(self.amp, port)
+        expected_fixed_ips = [
+            {'ip_address': FIXED_IP1, 'subnet_cidr': SUBNET_CIDR,
+             'host_routes': [{'destination': DEST1, 'nexthop': NEXTHOP},
+                             {'destination': DEST2, 'nexthop': NEXTHOP}]},
+            {'ip_address': FIXED_IP2, 'subnet_cidr': SUBNET_CIDR,
+             'host_routes': [{'destination': DEST1, 'nexthop': NEXTHOP},
+                             {'destination': DEST2, 'nexthop': NEXTHOP}]}
+        ]
+        self.driver.client.plug_network.assert_called_once_with(
+            self.amp, dict(mac_address=FAKE_MAC_ADDRESS,
+                           fixed_ips=expected_fixed_ips))
 
     def test_get_vrrp_interface(self):
         self.driver.get_vrrp_interface(self.amp)
@@ -172,9 +229,18 @@ class TestAmphoraAPIClientTest(base.TestCase):
         self.driver = driver.AmphoraAPIClient()
         self.base_url = "https://127.0.0.1:9443/0.5"
         self.amp = models.Amphora(lb_network_ip='127.0.0.1', compute_id='123')
-        self.port_info = dict(mac_address='123')
+        self.port_info = dict(mac_address=FAKE_MAC_ADDRESS)
+        # Override with much lower values for testing purposes..
+        conf = oslo_fixture.Config(cfg.CONF)
+        conf.config(group="haproxy_amphora", connection_max_retries=2)
 
-    def test_request(self):
+        self.subnet_info = {'subnet_cidr': FAKE_CIDR,
+                            'gateway': FAKE_GATEWAY,
+                            'mac_address': FAKE_MAC_ADDRESS,
+                            'vrrp_ip': self.amp.vrrp_ip}
+
+    @mock.patch('octavia.amphorae.drivers.haproxy.rest_api_driver.time.sleep')
+    def test_request(self, mock_sleep):
         self.assertRaises(driver_except.TimeOutException,
                           self.driver.request,
                           'get', self.amp, 'unavailableURL')
@@ -686,7 +752,7 @@ class TestAmphoraAPIClientTest(base.TestCase):
         m.post("{base}/plug/vip/{vip}".format(
             base=self.base_url, vip=FAKE_IP)
         )
-        self.driver.plug_vip(self.amp, FAKE_IP, FAKE_SUBNET_INFO)
+        self.driver.plug_vip(self.amp, FAKE_IP, self.subnet_info)
         self.assertTrue(m.called)
 
     @requests_mock.mock()
@@ -716,7 +782,7 @@ class TestAmphoraAPIClientTest(base.TestCase):
     @requests_mock.mock()
     def test_get_interface(self, m):
         interface = [{"interface": "eth1"}]
-        ip_addr = '10.0.0.1'
+        ip_addr = '192.51.100.1'
         m.get("{base}/interface/{ip_addr}".format(base=self.base_url,
                                                   ip_addr=ip_addr),
               json=interface)
