@@ -51,7 +51,7 @@ class TestException(Exception):
         return repr(self.value)
 
 
-@mock.patch('stevedore.driver.DriverManager.driver')
+@mock.patch('octavia.common.utils.get_network_driver')
 class TestNetworkTasks(base.TestCase):
     def setUp(self):
         network_tasks.LOG = mock.MagicMock()
@@ -65,12 +65,13 @@ class TestNetworkTasks(base.TestCase):
         self.amphora_mock.compute_id = COMPUTE_ID
         self.amphora_mock.status = constants.AMPHORA_ALLOCATED
         conf = oslo_fixture.Config(cfg.CONF)
-        conf.config(group="controller_worker", amp_network='netid')
+        conf.config(group="controller_worker", amp_boot_network_list=['netid'])
 
         super(TestNetworkTasks, self).setUp()
 
-    def test_calculate_delta(self,
-                             mock_driver):
+    def test_calculate_delta(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
         EMPTY = {}
         empty_deltas = {self.amphora_mock.id: data_models.Delta(
             amphora_id=self.amphora_mock.id,
@@ -87,19 +88,14 @@ class TestNetworkTasks(base.TestCase):
 
         self.amphora_mock.load_balancer = self.load_balancer_mock
         self.load_balancer_mock.amphorae = [self.amphora_mock]
-        self.load_balancer_mock.listeners = None
-        self.assertEqual({self.amphora_mock.id: None},
-                         net.execute(self.load_balancer_mock))
+        self.load_balancer_mock.pools = []
 
-        listener_mock = mock.MagicMock()
-        self.load_balancer_mock.listeners = [listener_mock]
-        listener_mock.default_pool = None
         self.assertEqual(empty_deltas, net.execute(self.load_balancer_mock))
         mock_driver.get_plugged_networks.assert_called_once_with(COMPUTE_ID)
 
         pool_mock = mock.MagicMock()
-        listener_mock.default_pool = pool_mock
-        pool_mock.members = None
+        self.load_balancer_mock.pools = [pool_mock]
+        pool_mock.members = []
         self.assertEqual(empty_deltas, net.execute(self.load_balancer_mock))
 
         member_mock = mock.MagicMock()
@@ -143,8 +139,9 @@ class TestNetworkTasks(base.TestCase):
         self.assertEqual({self.amphora_mock.id: ndm},
                          net.execute(self.load_balancer_mock))
 
-    def test_get_plumbed_networks(self,
-                                  mock_driver):
+    def test_get_plumbed_networks(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
         mock_driver.get_plugged_networks.side_effect = [['blah']]
         net = network_tasks.GetPlumbedNetworks()
 
@@ -152,8 +149,9 @@ class TestNetworkTasks(base.TestCase):
         mock_driver.get_plugged_networks.assert_called_once_with(
             COMPUTE_ID)
 
-    def test_plug_networks(self,
-                           mock_driver):
+    def test_plug_networks(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
 
         def _interface(network_id):
             return [data_models.Interface(network_id=network_id)]
@@ -208,7 +206,10 @@ class TestNetworkTasks(base.TestCase):
                           delta)
         mock_driver.unplug_network.assert_called_once_with(COMPUTE_ID, 1)
 
-    def test_unplug_networks(self, mock_driver):
+    def test_unplug_networks(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
+
         def _interface(network_id):
             return [data_models.Interface(network_id=network_id)]
 
@@ -242,7 +243,10 @@ class TestNetworkTasks(base.TestCase):
         net.execute(self.amphora_mock, delta)  # No exception
         mock_driver.unplug_network.assert_called_once_with(COMPUTE_ID, 1)
 
-    def test_get_member_ports(self, mock_driver):
+    def test_get_member_ports(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
+
         def _interface(port_id):
             return [data_models.Interface(port_id=port_id)]
 
@@ -274,7 +278,10 @@ class TestNetworkTasks(base.TestCase):
         mock_driver.get_subnet.assert_called_once_with(1)
         self.assertEqual([port_mock], ports)
 
-    def test_handle_network_delta(self, mock_driver):
+    def test_handle_network_delta(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
+
         def _interface(network_id):
             return [data_models.Interface(network_id=network_id)]
 
@@ -351,8 +358,9 @@ class TestNetworkTasks(base.TestCase):
         net.execute({self.amphora_mock.id: delta})
         mock_driver.unplug_network.assert_called_once_with(COMPUTE_ID, 1)
 
-    def test_plug_vip(self,
-                      mock_driver):
+    def test_plug_vip(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
         net = network_tasks.PlugVIP()
 
         mock_driver.plug_vip.return_value = ["vip"]
@@ -371,13 +379,17 @@ class TestNetworkTasks(base.TestCase):
         net.revert(["vip"], LB)
         mock_driver.unplug_vip.assert_called_once_with(LB, LB.vip)
 
-    def test_unplug_vip(self, mock_driver):
+    def test_unplug_vip(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
         net = network_tasks.UnplugVIP()
 
         net.execute(LB)
         mock_driver.unplug_vip.assert_called_once_with(LB, LB.vip)
 
-    def test_allocate_vip(self, mock_driver):
+    def test_allocate_vip(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
         net = network_tasks.AllocateVIP()
 
         mock_driver.allocate_vip.return_value = LB.vip
@@ -398,64 +410,46 @@ class TestNetworkTasks(base.TestCase):
         net.revert(vip_mock, LB)
         mock_driver.deallocate_vip.assert_called_once_with(vip_mock)
 
-    def test_deallocate_vip(self, mock_driver):
+    def test_deallocate_vip(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
         net = network_tasks.DeallocateVIP()
         vip = o_data_models.Vip()
         lb = o_data_models.LoadBalancer(vip=vip)
         net.execute(lb)
         mock_driver.deallocate_vip.assert_called_once_with(lb.vip)
 
-    def test_update_vip(self, mock_driver):
+    def test_update_vip(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
         vip = o_data_models.Vip()
         lb = o_data_models.LoadBalancer(vip=vip)
         net_task = network_tasks.UpdateVIP()
         net_task.execute(lb)
         mock_driver.update_vip.assert_called_once_with(lb)
 
-    def test_get_amphorae_network_configs(self, mock_driver):
-
+    def test_get_amphorae_network_configs(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
+        lb = o_data_models.LoadBalancer()
         net_task = network_tasks.GetAmphoraeNetworkConfigs()
-        net_task.execute(LB)
-        mock_driver.get_subnet.assert_called_once_with(SUBNET_ID)
-        mock_driver.get_port.assert_called_once_with(PORT_ID)
+        net_task.execute(lb)
+        mock_driver.get_network_configs.assert_called_once_with(lb)
 
-        mock_driver.reset_mock()
-        self.amphora_mock.status = constants.DELETED
-        self.load_balancer_mock.amphorae = [self.amphora_mock]
-        net_task = network_tasks.GetAmphoraeNetworkConfigs()
-        configs = net_task.execute(self.load_balancer_mock)
-        self.assertEqual({}, configs)
-
-        mock_driver.reset_mock()
-        self.vip_mock.port_id = 1
-        self.amphora_mock.status = constants.ACTIVE
-        self.amphora_mock.vrrp_port_id = 2
-        self.amphora_mock.vrrp_ip = "10.0.0.1"
-        self.amphora_mock.ha_port_id = 3
-        self.amphora_mock.ha_ip = "10.0.0.2"
-        self.load_balancer_mock.amphorae = [self.amphora_mock]
-        net_task = network_tasks.GetAmphoraeNetworkConfigs()
-        configs = net_task.execute(self.load_balancer_mock)
-        vip_port_id = mock.call(1)
-        vrrp_port_id_call = mock.call(2)
-        ha_port_id_call = mock.call(3)
-        vrrp_port_subnet_call = mock.call().get_subnet_id('10.0.0.1')
-        ha_port_subnet_call = mock.call().get_subnet_id('10.0.0.2')
-        mock_driver.get_port.assert_has_calls([vip_port_id,
-                                               vrrp_port_id_call,
-                                               vrrp_port_subnet_call,
-                                               ha_port_id_call,
-                                               ha_port_subnet_call])
-        self.assertEqual(1, len(configs))
-
-    def test_failover_preparation_for_amphora(self, mock_driver):
+    def test_failover_preparation_for_amphora(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
         failover = network_tasks.FailoverPreparationForAmphora()
         amphora = o_data_models.Amphora(id=AMPHORA_ID,
                                         lb_network_ip=IP_ADDRESS)
         failover.execute(amphora)
         mock_driver.failover_preparation.assert_called_once_with(amphora)
 
-    def test_retrieve_portids_on_amphora_except_lb_network(self, mock_driver):
+    def test_retrieve_portids_on_amphora_except_lb_network(
+            self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
+
         def _interface(port_id):
             return [data_models.Interface(port_id=port_id)]
 
@@ -497,7 +491,10 @@ class TestNetworkTasks(base.TestCase):
         ports = net_task.execute(amphora)
         self.assertEqual(1, len(ports))
 
-    def test_plug_ports(self, mock_driver):
+    def test_plug_ports(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
+
         amphora = mock.MagicMock()
         port1 = mock.MagicMock()
         port2 = mock.MagicMock()
@@ -507,3 +504,31 @@ class TestNetworkTasks(base.TestCase):
 
         mock_driver.plug_port.assert_any_call(amphora, port1)
         mock_driver.plug_port.assert_any_call(amphora, port2)
+
+    def test_plug_vip_port(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
+        vrrp_port = mock.MagicMock()
+
+        amphorae_network_config = mock.MagicMock()
+        amphorae_network_config.get().vrrp_port = vrrp_port
+
+        plugvipport = network_tasks.PlugVIPPort()
+        plugvipport.execute(self.amphora_mock, amphorae_network_config)
+        mock_driver.plug_port.assert_any_call(self.amphora_mock, vrrp_port)
+
+        # test revert
+        plugvipport.revert(None, self.amphora_mock, amphorae_network_config)
+        mock_driver.unplug_port.assert_any_call(self.amphora_mock, vrrp_port)
+
+    def test_wait_for_port_detach(self, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_driver
+
+        amphora = o_data_models.Amphora(id=AMPHORA_ID,
+                                        lb_network_ip=IP_ADDRESS)
+
+        waitforportdetach = network_tasks.WaitForPortDetach()
+        waitforportdetach.execute(amphora)
+
+        mock_driver.wait_for_port_detach.assert_called_once_with(amphora)
